#!/bin/env dls-python
import PandATest
from collections import OrderedDict


class Encoder(PandATest.PandATest):

    def __init__(self, *args, **kwargs):
        super(Encoder, self).__init__(*args, **kwargs)

    def test_code(self):

        #(cmd, (expected result, +- bound of result))
        setup_quad_commands = OrderedDict([
            ('INENC{}.BYPASS=On'.format(self.blocknum),            ('OK', 0)),
            ('OUTENC{}.BYPASS=On'.format(self.blocknum),           ('OK', 0)),
            ('INENC{}.PROTOCOL=Quadrature'.format(self.blocknum),  ('OK', 0)),
            ('OUTENC{}.PROTOCOL=Quadrature'.format(self.blocknum), ('OK', 0))
        ])
        setup_ssi_commands = OrderedDict([
            ('INENC{}.PROTOCOL=SSI'.format(self.blocknum),  ('OK', 0)),
            ('OUTENC{}.PROTOCOL=SSI'.format(self.blocknum), ('OK', 0))
        ])
        quad_fields = ['A', 'B', 'Z']
        q_ch_connect = {
            quad_fields[0]: quad_fields[0],
            quad_fields[1]: quad_fields[1],
            quad_fields[2]: quad_fields[2],
        }
        ssi_fields = ['CLK', 'DATA']

        #SET FOR QUADRATURE
        for cmd, expected in setup_quad_commands.items():
            response = self.send_cmd(cmd)
            self.test_result.append(
                self.check_response(response, expected[0], cmd, expected[1]))

        default = 0
        for outp in quad_fields:
            for val in [1, 0]:
                out_cmd = 'OUTENC{}.{}={}'.format(self.blocknum, outp, self.toggle_cmd[val])
                self.set_output(out_cmd)
                in_cmd = 'INENC{}.{}?'.format(self.blocknum, '{}')
                self.toggle_check(in_cmd, outp, quad_fields, q_ch_connect[outp], default, val)

        #SET FOR SSI
        for cmd, expected in setup_ssi_commands.items():
            response = self.send_cmd(cmd)
            self.test_result.append(
                self.check_response(response, expected[0], cmd, expected[1]))

        #TOGGLE SSI FIELDS
        ok_string = {
            0: "..NOT OK",
            1: "..OK"
        }

        for val in [1, 0]:
            out_cmd = 'INENC{}.CLK={}'.format(self.blocknum, self.toggle_cmd[val])
            self.set_output(out_cmd)
            in_cmd = 'OUTENC{}.{}?'.format(self.blocknum, '{}')
            self.toggle_check(in_cmd, '.CLK', ['CLK'], 'CLK', default, val)

        for val in [1, 0]:
            out_cmd = 'OUTENC{}.DATA={}'.format(self.blocknum, self.toggle_cmd[val])
            self.set_output(out_cmd)
            in_cmd = 'INENC{}.{}?'.format(self.blocknum, '{}')
            self.toggle_check(in_cmd, '.DATA', ['DATA'], 'DATA', default, val)

        return not all(self.test_result)