#!/bin/env dls-python
import logging


class PandATest(object):
    def __init__(self, socket, blocknum=0):
        self.socket = socket
        self.logger = self.setup_logging()
        self.test_result = []
        self.blocknum = blocknum
        self.ok_string = {
            0: ".. NOT OK",
            1: ".. OK"
        }
        self.toggle_cmd = {
            0: 'BITS.ZERO',
            1: 'BITS.ONE'
        }

    def setup_logging(self):
        name = '.'.join([self.__class__.__name__])
        root_logger = logging.getLogger(name)
        return root_logger

    def run(self):
        self.get_user_input()
        self.logger.info("TEST START")
        self.test_code()

    def send_cmd(self, cmd):
        self.socket.sendall(cmd + '\n')
        result = self.socket.recv(4096)
        return result.strip('\n')

    def test_code(self):
        pass

    def set_output(self, cmd):
        response = self.send_cmd(cmd)
        if response.strip('\n') != 'OK':
            self.logger.info("ERROR SENDING: %s", cmd)

    def get_summary(self):
        error_count = self.test_result.count(0)
        summary = ["{:<10}{:<10} ERRORS: {}".format(
                                                self.__class__.__name__,
                                                self.blocknum,
                                                error_count)
        ]
        return summary

    def get_user_input(self):
        pass

    def check_response(self, response, expected, cmd, tol=0):
        try:
            response_val = int(response.split('=')[1])
            expected_val = int(expected.split('=')[1])
            check = expected_val - tol <= response_val <= expected_val + tol
        except IndexError:
            check = response == expected

        if check:
            self.logger.info("{:<20} [{:<9}]  ..OK".format(cmd, response))
            return 1
        else:
            self.logger.info("{:<20} [{:<9}]  ..NOT OK".format(cmd, response))
            return 0

    def toggle_check(self, in_cmd, output, inputs, expected, default, tgl_val):
        for inp in inputs:
            cmd = in_cmd.format(inp)
            response = self.send_cmd(cmd)
            if expected == inp:
                if (response) == 'OK ={}'.format(tgl_val):
                    self.test_result.append(1)
                else:
                    self.test_result.append(0)
            else:
                if (response) == 'OK ={}'.format(default):
                    self.test_result.append(1)
                else:
                    self.test_result.append(0)
            self.logger.info("{:<21}[{:<9}]  {}".format(
                "OUT{}=>{}, INP{}=>".format(output, tgl_val, inp),
                response, self.ok_string[self.test_result[-1]]))